<?php
require_once("BaseController.php");
use IvozProvider\Mapper\Sql as Mapper;
use Agi\Action\DDIAction;
use Agi\Action\ExtensionAction;
use Agi\Action\UserCallAction;
use Agi\Action\HuntGroupAction;
use Agi\Action\IVRAction;
use Agi\Action\ServiceAction;
use Agi\Action\FriendCallAction;
use Agi\Action\RetailCallAction;
use Agi\Action\ExternalUserCallAction;
use Agi\Action\ExternalFriendCallAction;
use Agi\Action\ExternalRetailCallAction;
use Agi\Action\QueueAction;

/**
 * @brief Controller for Incoming and Outgoing calls
 *
 * This controllers is invoked from different contexts of dialplan and routes
 * the call based on configuretion
 *
 * Following actions are defined in this controller
 * - incoming: Handle incoming calls from external numbers
 * - outgoing: Handle outgoing calls from registered users
 * - forwards: Handle channel redirections and transfers
 * - dialstatus: Handle post-call user call forwards
 *
 * @package AGI
 * @subpackage CallsController
 * @author Gaizka Elexpe <gaizka@irontec.com>
 * @author Ivan Alonso [kaian] <kaian@irontec.com>
 */
class CallsController extends BaseController
{
    /**
     * @brief Incomming from from external numbers
     */
    public function trunksAction ()
    {
        // Get Dialed number
        $exten = $this->agi->getExtension();

        // Check if incoming DDI is for us
        $DDIMapper = new Mapper\DDIs();
        $ddi = $DDIMapper->findOneByField("DDIE164", $exten);
        if (empty($ddi)) {
            $this->agi->error("DDI %s not found in database.", $exten);
            return;
        }

        // Store Original E164 Number for further transformations
        $this->agi->setOrigCallerIdNum($this->agi->getCallerIdNum());

        // Mark this call as external
        $this->agi->setCallType("external");

        // Remove any Diversion header from external calls
        if ($this->agi->getRedirecting('count')) {
            $this->agi->setRedirecting('count', 0);
        }

        // Set Outgoing Channels X-CID header variable
        $this->agi->setVariable("__CALL_ID", $this->agi->getCallId());

        // Get company MusicClass: company, Generic or default
        $company = $ddi->getCompany();
        $this->agi->setVariable("__COMPANYID", $company->getId());
        $this->agi->setVariable("CHANNEL(musicclass)", $company->getMusicClass());
        $this->agi->setVariable("CHANNEL(language)", $ddi->getLanguageCode());

        // Check company On demand record code
        if ($company->getOnDemandRecord()) {
            $this->agi->setVariable("FEATUREMAP(automixmon)", $company->getOnDemandRecordDTMFs());
        }

        // Set DDI as the caller
        $this->agi->setChannelCaller($ddi);

        // Process this DDI
        $ddiAction = new DDIAction($this);
        $ddiAction
            ->setDDI($ddi)
            ->process();
    }

    /**
     * @brief Outgoing calls from terminals to Extensions, Services o World
     */
    public function usersAction ()
    {
        /**
         * Determine who is placing this call:
         * - SIPTRANSFER: set by asterisk on blind transfers
         * - Diversion:   set by asterisk on 302 Moved SIP Message
         * - Endpoint:    set by asterisk on matching endpoint
         */
        if ($this->agi->getVariable("SIPTRANSFER")) {
            // Asterisk stores the Refered-By header of the transferer
            $transfererURI = $this->agi->extractURI($this->agi->getVariable("SIPREFERREDBYHDR"), "uri");
            $aorMapper = new Mapper\AstPsAors;
            // Get the endpoint name matching the referer contact
            $endpointName = $aorMapper->getSorceryByContact($transfererURI);
            // Transferer is the new caller
            if ($user = $this->getEntityFromEndpoint($endpointName)) {
                $this->agi->setChannelCaller($user);
                // Set Caller extension in Referred header
                $transfererURI = str_replace($endpointName, $user->getExtensionNumber(), $transfererURI);
                $this->agi->setVariable("__SIPREFERREDBYHDR", $transfererURI);
            }
        } else if ($forwarder = $this->agi->getRedirecting('from-num')) {
            // 302 Moved here caller. The variable MUST store the last dialed endpoint
            $endpointName = $this->agi->getVariable("DIAL_ENDPOINT");
            // Forwarder is the new caller
            if ($user = $this->getEntityFromEndpoint($endpointName)) {
                $this->agi->setChannelCaller($user);
            }
            // Remove any Diversion header generated by Terminals (they will be added if required later)
            $this->agi->setRedirecting('count', 0);
        } else {
            $endpointName = $this->agi->getEndpoint();
            // User terminal calling, set caller and origin
            if ($user = $this->getEntityFromEndpoint($endpointName)) {
                $this->agi->setChannelCaller($user);
                $this->agi->setChannelOrigin($user);
            }
        }

        // Do we get who is actually calling?
        if (empty($user)) {
            $this->agi->error("Call without valid endpointName. Dropping.");
            return;
        }

        // Set Company/Brand/Generic Music class
        $company = $user->getCompany();
        $this->agi->setVariable("__COMPANYID", $company->getId());

        // Mark this call as generated from user
        $this->agi->setCallType("internal");

        // Set Outgoing Channels X-CID header variable
        $this->agi->setVariable("__CALL_ID", $this->agi->getCallId());

        // Set user language and music
        $this->agi->setVariable("CHANNEL(language)", $user->getLanguageCode());
        $this->agi->setVariable("CHANNEL(musicclass)", $company->getMusicClass());

        // Check company On demand record code
        if ($company->getOnDemandRecord()) {
            $this->agi->setVariable("FEATUREMAP(automixmon)", $company->getOnDemandRecordDTMFs());
        }

        // Remove any Diversion header generated by Terminals (they will be added if required later)
        if ($this->agi->getRedirecting('count')) {
            $this->agi->setRedirecting('count', 0);
        }

        // Check User's permission to does this call
        $exten = $this->agi->getExtension();

        // Some feedback for asterisk cli
        $this->agi->notice("Processing outgoing call from \e[0;32m%s [user%d]\e[0;93m to number %s",
                        $user->getFullName(), $user->getId(), $exten);

        // Check if this extension starts with '*' code
        if (strpos($exten, '*') === 0) {
            if (($service = $company->getService($exten))) {
                $this->agi->verbose("Number %s belongs to a company service [service%d].",
                                $exten, $service->getId());

                // Handle service code
                $serviceAction = new ServiceAction($this);
                $serviceAction
                    ->setService($service)
                    ->process();

            } else {
                // Decline this call if not matching service is found
                $this->agi->verbose("Invalid Service code %s for comany %d", $exten, $company->getId());
                $this->agi->hangup();
            }

        // Check if this is an extension call
        } elseif (($dstExtension = $company->getExtension($exten))) {
            $this->agi->verbose("Number %s belongs to a Company Extension [extension%d].",
                            $exten, $dstExtension->getId());

            // Update Diversion Header with User Extension Number
            if (isset($forwarder) && !empty($forwarder)) {
                $this->agi->setRedirecting('from-num,i', $user->getExtensionNumber());
                $this->agi->setRedirecting('from-tag,i', $user->getExtensionNumber());
                $this->agi->setRedirecting('count,i', 1);
            }

            // Handle extension
            $extensionAction = new ExtensionAction($this);
            $extensionAction
                ->setExtension($dstExtension)
                ->process();

        // Check if this number matches one of friendly trunks patterns
        } else if (($friend = $company->getFriend($exten))) {
            $this->agi->verbose("Number %s is handled by friendly trunk.", $exten);

            // Update Diversion Header with User Extension Number
            if (isset($forwarder) && !empty($forwarder)) {
                $this->agi->setRedirecting('from-num,i', $user->getExtensionNumber());
                $this->agi->setRedirecting('from-tag,i', $user->getExtensionNumber());
                $this->agi->setRedirecting('count,i', 1);
            }

            // Handle call through friendly trunk
            $friendAction = new FriendCallAction($this);
            $friendAction
                ->setFriend($friend)
                ->setDestination($exten)
                ->call();

        // This number don't belong to IvozProvider
        } else {
            $this->agi->verbose("Number %s is handled as external number.", $exten);

            // Update Diversion Header with User Outgoing DDI
            if (isset($forwarder) && !empty($forwarder)) {
                $this->agi->setRedirecting('from-name,i', $user->getFullName());
                $this->agi->setRedirecting('from-num,i',  $user->getOutgoingDDINumber());
                $this->agi->setRedirecting('from-tag,i',  $user->getExtensionNumber());
                $this->agi->setRedirecting('count,i', 1);
            }

            // Otherwise, handle this call as external
            $externalCallAction = new ExternalUserCallAction($this);
            $externalCallAction
                ->setDestination($exten)
                ->process();
        }
    }

    /**
     * @brief Process User after call status
     */
    public function userstatusAction ()
    {
        // Get the called endpoint to check postcall actions
        $endpointName = $this->agi->getVariable("DIAL_ENDPOINT");
        // Get user from the endpoint.
        $user = $this->getEntityFromEndpoint($endpointName);

        // Get user from the terminal.
        if (empty($user)) {
            $this->agi->error("Terminal %s has no user (BUG?).", $endpointName);
            return;
        }

        // ProcessDialStatus
        $userAction = new UserCallAction($this);
        $userAction
            ->setUser($user)
            ->processDialStatus();
    }

    /**
     * @brief Process Retail after call status
     */
    public function retailstatusAction ()
    {
        // Get the called endpoint to check postcall actions
        $endpointName = $this->agi->getVariable("DIAL_ENDPOINT");
        // Get retailAccount from the endpoint.
        $retailAccount = $this->getEntityFromEndpoint($endpointName);
        // Restore CallerId with original trunks E.164 number
        $this->agi->setCallerIdNum($this->agi->getOrigCallerIdNum());

        // Get user from the terminal.
        if (empty($retailAccount)) {
            $this->agi->error("Endpoint %s has no retail account (BUG?).", $endpointName);
            return;
        }

        // ProcessDialStatus
        $retailAction = new RetailCallAction($this);
        $retailAction
            ->setRetailAccount($retailAccount)
            ->processDialStatus();
    }

    /**
     * @brief Outgoing calls from queues
     */
    public function queuesAction()
    {
        $companyId = $this->agi->getVariable("COMPANYID");
        $companyMapper = new Mapper\Companies();
        $company = $companyMapper->find($companyId);

        if (empty($company)) {
            $this->agi->error("No company found with id %d (BUG?).", $companyId);
            return;
        }

        $queueMemberId = $this->agi->getExtension();
        $queueMembersMapper = new Mapper\QueueMembers();
        $queueMember = $queueMembersMapper->find($queueMemberId);
        if (empty($queueMember)) {
           $this->agi->error("Queue member with id %d does not exists.", $queueMemberId);
        }

        $user = $queueMember->getUser();
        if (empty($user)) {
            $this->agi->error("No user found for queue member %d", $queueMemberId);
            return;
        }

        $endpoint = $user->getEndpoint();
        if (empty($endpoint)) {
            $this->agi->error("User %d has no endpoint associated", $user->getId());
            return;
        }

        // For external calls, transform the number to user preferred format
        if ($this->agi->getCallType() == "external") {
            // Transfor number to User Preferred
            $preferred = $user->E164ToPreferred($this->agi->getOrigCallerIdNum());
            $this->agi->setCallerIdNum($preferred);
        }

        $this->agi->setVariable("DIAL_OPTS", "ic");
        $this->agi->setVariable("DIAL_DST", "PJSIP/" . $endpoint->getSorceryId());
    }

    /**
     * @brief After Queue process
     */
    public function queuestatusAction()
    {
        $queueId = $this->agi->getVariable("QUEUE_ID");
        $queueMapper = new Mapper\Queues();
        $queue = $queueMapper->find($queueId);

        // Process Queue Timeout
        $queueAction = new QueueAction($this);
        $queueAction
            ->setQueue($queue);


        switch($this->agi->getVariable("QUEUESTATUS")) {
            case 'TIMEOUT':
                $queueAction->processTimeout();
                break;
            case 'FULL':
                $queueAction->processFull();
                break;
        }
    }

    /**
     * @brief Outgoing calls from friends
     */
    public function friendsAction ()
    {
        // Get identified Enpoint name
        $endpointName = $this->agi->getEndpoint();

        // Get friend from the endpoint.
        $friend = $this->getEntityFromEndpoint($endpointName);
        if (empty($friend)) {
            $this->agi->error("Endpoint %s does not belong to a friend.", $endpointName);
            return;
        }

        // Set Company/Brand/Generic Music class
        $company = $friend->getCompany();
        $this->agi->setVariable("__COMPANYID", $company->getId());

        // Check User's permission to does this call
        $exten = $this->agi->getExtension();

        // Mark this call as generated from user
        $this->agi->setCallType("internal");

        // Set Outgoing Channels X-CID header variable
        $this->agi->setVariable("_CALL_ID", $this->agi->getCallId());

        // Set user language and music
        $this->agi->setVariable("CHANNEL(language)",   $friend->getLanguageCode());
        $this->agi->setVariable("CHANNEL(musicclass)", $company->getMusicClass());

        // Set Friend as the caller
        $this->agi->setChannelCaller($friend);
        $this->agi->setChannelOrigin($friend);

        // Some feedback for asterisk cli
        $this->agi->notice("Processing outgoing call from \e[0;36m%s [friend%d]\e[0;93m to number %s",
                        $friend->getName(), $friend->getId(), $exten);

        // Check if this is an extension call
        if (($dstExtension = $company->getExtension($exten))) {
            $this->agi->verbose("Number %s belongs to a Company Extension [extension%d].",
                            $exten, $dstExtension->getId());

            // Handle extension
            $extensionAction = new ExtensionAction($this);
            $extensionAction
                ->setExtension($dstExtension)
                ->process();

        } else if (($outfriend = $company->getFriend($exten))) {
            $this->agi->verbose("Number %s is handled by friendly trunk.", $exten);

            // Handle call through friendly trunk
            $friendAction = new FriendCallAction($this);
            $friendAction
                ->setFriend($outfriend)
                ->setDestination($exten)
                ->call();

        // This number don't belong to IvozProvider
        } else {
            $this->agi->verbose("Number %s is handled as external number.", $exten);

            // Otherwise, handle this call as external
            $externalCallAction = new ExternalFriendCallAction($this);
            $externalCallAction
                ->setDestination($exten)
                ->process();
        }
    }

    /**
     * @brief Outgoing calls from friends
     */
    public function retailAction ()
    {
        // Get identified Enpoint name
        $endpointName = $this->agi->getEndpoint();

        // Get retail from the endpoint.
        $retail = $this->getEntityFromEndpoint($endpointName);
        if (empty($retail)) {
            $this->agi->error("Endpoint %s is not a valid retail endpoint.", $endpointName);
            return;
        }

        // Set Company/Brand/Generic Music class
        $company = $retail->getCompany();
        $this->agi->setVariable("__COMPANYID", $company->getId());
        $this->agi->setVariable("CHANNEL(language)", $company->getLanguageCode());

        // Get call destination
        $exten = $this->agi->getExtension();

        // Set Outgoing Channels X-CID header variable
        $this->agi->setVariable("_CALL_ID", $this->agi->getCallId());

        // Set User as the caller
        $this->setChannelOwner($retail);

        // Check if this extension starts with '*' code
        if (strpos($exten, '*') === 0) {
            if (($service = $company->getBrand()->getVoicemailService())) {
                if ($service->getCode() == substr($exten, 1)) {
                    $this->agi->verbose("Number %s belongs to a brand service [service%d].", $exten, $service->getId());

                    // Handle service code
                    $serviceAction = new ServiceAction($this);
                    $serviceAction
                        ->setService($service)
                        ->process();

                    return;
                }
            }

            // Decline this call if not matching service is found
            $this->agi->verbose("Invalid Service code %s for brand %d", $exten, $company->getBrandId());
            $this->agi->hangup();
        } else {
            // Some feedback for asterisk cli
            $this->agi->notice("Processing outgoing call from Retail account \e[0;36m%s [retail%d]\e[0;93m to number %s",
                $retail->getName(), $retail->getId(), $exten);

            // Otherwise, handle this call as external
            $externalCallAction = new ExternalRetailCallAction($this);
            $externalCallAction
                ->setDestination($exten)
                ->process();
        }
    }

    /**
     * @brief Incoming calls to conference
     */
    public function conferencesAction()
    {
        $conferenceId = $this->agi->getExtension();
        $conferenceMapper = new Mapper\ConferenceRooms();
        $conference = $conferenceMapper->find($conferenceId);
        if (empty($conference)) {
            $this->agi->error("Conference %d not found in database", $conferenceId);
            return;
        }

        // Get company from conference
        $company = $conference->getCompany();

        // Set desired channel language
        $this->agi->setVariable("CHANNEL(language)", $this->agi->getSIPHeader("X-Info-Conf-Lang"));

        // Set user language and music
        $this->agi->setVariable("CHANNEL(musicclass)", $company->getMusicClass());

        // Check if conference requires pin
        if ($conference->getPinProtected()) {
           $this->agi->setConferenceSetting('user,pin', $conference->getPinCode());
        }

        // Check if conference has max members
        if ($conference->getMaxMembers()) {
            $this->agi->setConferenceSetting('bridge,max_members', $conference->getMaxMembers());
        }

        // Enable video support
        $this->agi->setConferenceSetting('bridge,video_mode', 'follow_talker');
    }

    /**
     * @brief Process IVR after call status
     */
    public function ivrstatusAction ()
    {
        $dialStatus = $this->agi->getVariable("DIALSTATUS");

        // Noone picked up
        if ($dialStatus == "NOANSWER") {
            $ivrId = $this->agi->getVariable("IVRID");
            $ivrType = $this->agi->getVariable("IVRTYPE");

            if ($ivrType == 'COMMON') {
                $ivrMapper = new Mapper\IVRCommon();
            } else {
                $ivrMapper = new Mapper\IVRCustom();
            }

            // Get IVR data
            $ivr = $ivrMapper->find($ivrId);

            // Process NoAnswer handler
            $ivrAction = new IVRAction($this);
            $ivrAction
                ->setIvr($ivr)
                ->processTimeout();
        }
    }

    /**
     * @brief Call a user from a Huntgroup
     */
    public  function hgcalluserAction()
    {
        // Get running Huntgroup
        $huntgroupId = $this->agi->getVariable("HG_ID");
        $huntgroupMapper = new Mapper\HuntGroups();
        $huntgroup = $huntgroupMapper->find($huntgroupId);

        // Continue processing
        $hungroupAction = new HuntGroupAction($this);
        $hungroupAction
            ->setHuntGroup($huntgroup)
            ->call();
    }

    /**
     * @brief Process Huntgroup after call status
     */
    public function hgstatusAction ()
    {
        // Get running Huntgroup
        $huntgroupId = $this->agi->getVariable("HG_ID");
        $huntgroupMapper = new Mapper\HuntGroups();
        $huntgroup = $huntgroupMapper->find($huntgroupId);

        // Continue processing
        $hungroupAction = new HuntGroupAction($this);
        $hungroupAction
            ->setHuntGroup($huntgroup)
            ->processHuntgroupStatus();
    }

    /**
     * @brief Add SIP Headers for proxies
     */
    public function addheadersAction()
    {
        $companyId = $this->agi->getVariable("COMPANYID");
        $companyMapper = new Mapper\Companies();
        $company = $companyMapper->find($companyId);

        // Add headers for Friendly Kamailio  Proxy;-))
        $this->agi->setSIPHeader("X-Call-Id",            $this->agi->getVariable("CALL_ID"));
        $this->agi->setSIPHeader("X-Info-BrandId",       $company->getBrandId());
        $this->agi->setSIPHeader("X-Info-CompanyId",     $company->getId());
        $this->agi->setSIPHeader("X-Info-CompanyName",   $company->getName());
        $this->agi->setSIPHeader("X-Info-MediaRelaySet", $company->getMediaRelaySetsId());

        // Get Calle data, take if from called endpoint
        $endpointsMapper = new Mapper\AstPsEndpoints();
        $endpoint = $endpointsMapper->findOneByField("sorcery_id", $this->agi->getEndpoint());
        if (!empty($endpoint)) {
            $terminal = $endpoint->getTerminal();
            if (!is_null($terminal)) {
                $this->agi->setSIPHeader("X-Info-Callee", $terminal->getUser()->getExtensionNumber());
                $this->agi->setSIPHeader("X-Info-UserMaxCalls", $terminal->getUser()->getMaxCalls());
            }
            $friend = $endpoint->getFriend();
            if (!is_null($friend)) {
                $exten = $this->agi->getExtension();
                $this->agi->setSIPHeader("X-Info-Callee", $exten);
                $this->agi->setSIPHeader("X-Info-Friend", $friend->getRequestURI($exten));
                $this->agi->setSIPHeader("X-Info-UserMaxCalls", 0);
            }
            $retail = $endpoint->getRetailAccount();
            if (!is_null($retail)) {
                $exten = $this->agi->getExtension();
                $this->agi->setSIPHeader("X-Info-Callee", $exten);
                $this->agi->setSIPHeader("X-Info-Retail", $retail->getRequestURI($exten));
                $this->agi->setSIPHeader("X-Info-UserMaxCalls", 0);
            }

            // Set on-demand recording header (only for proxyusers)
            if ($company->getOnDemandRecord()) {
                $this->agi->setSIPHeader("X-Info-RecordCode", $company->getOnDemandRecordCode());
                $this->agi->setVariable("FEATUREMAP(automixmon)", $company->getOnDemandRecordDTMFs());
            }

        } else {
            $this->agi->setSIPHeader("X-Info-CompanyDomain", $company->getDomain());

            // Set special headers for Fax outgoing calls
            if ($this->agi->getVariable("FAXFILE_ID")) {
                $this->agi->setSIPHeader("X-Info-Special", "fax");
            }

            // Add maxcalls headers for KamTrunks
            $this->agi->setSIPHeader("X-Info-BrandMaxCalls", $company->getBrand()->getMaxCalls());
            $this->agi->setSIPHeader("X-Info-CompanyMaxCalls", $company->getMaxCalls());
        }

        // Set Special header for Forwarding
        if ($this->agi->getRedirecting('from-tag')) {
            $this->agi->setSIPHeader("X-Info-ForwardExt", $this->agi->getRedirecting('from-tag'));
        }

        // Set recording header
        if ($this->agi->getVariable("RECORD")) {
            $this->agi->setSIPHeader("X-Info-Record", $this->agi->getVariable("RECORD"));
        }

        // Request intra DDI bounce
        if ($this->agi->getVariable("BOUNCEME")) {
            $this->agi->setSIPHeader("X-Info-BounceMe", $this->agi->getVariable("BOUNCEME"));
        }

        // Set pickups group on outgoing channels
        if ($this->agi->getVariable("CHANNEL(namedpickupgroup)")) {
            $this->agi->setVariable("CHANNEL(namedcallgroup)", $this->agi->getVariable("CHANNEL(namedpickupgroup)"));
        }

        // Set conference options
        if ($this->agi->getVariable("CONFERENCE_ID")) {
            $this->agi->setSIPHeader("X-Info-Conf", $this->agi->getVariable("CONFERENCE_ID"));
            $this->agi->setSIPHeader("X-Info-Conf-Lang", $this->agi->getVariable("CONFERENCE_LANG"));
        }
    }

    public function updatelineAction()
    {
        $user = $this->agi->getChannelCaller();
        $e164exten = $this->agi->getExtension();
        $exten = $user->E164toPreferred($e164exten);
        $this->agi->setConnectedLine('num,i', $exten);
        $this->agi->setConnectedLine('name', '');
    }

    private function setChannelOwner($owner)
    {
        if ($owner instanceof \IvozProvider\Model\Raw\Users)
            $this->agi->setVariable("CALLER_TYPE", "USER");
        if ($owner instanceof \IvozProvider\Model\Raw\DDIs)
            $this->agi->setVariable("CALLER_TYPE", "DDI");
        if ($owner instanceof \IvozProvider\Model\Raw\Friends)
            $this->agi->setVariable("CALLER_TYPE", "FRIEND");
        if ($owner instanceof \IvozProvider\Model\Raw\RetailAccounts)
            $this->agi->setVariable("CALLER_TYPE", "RETAIL");
        $this->agi->setVariable("CALLER_ID", $owner->getId());
    }

    private function getChannelOwner()
    {
        switch($this->agi->getVariable("CALLER_TYPE")) {
            case "USER":
                $mapper = new Mapper\Users();
                break;
            case "DDI":
                $mapper = new Mapper\DDIs();
                break;
            case "FRIEND":
                $mapper = new Mapper\Friends();
                break;
            case "RETAIL":
                $mapper = new Mapper\RetailAccounts();
                break;
            default: return null;
        }

        return $mapper->find($this->agi->getVariable("CALLER_ID"));
    }


    public function getEntityFromEndpoint($endpointName)
    {
        // Get caller endpoint model
        $endpointsMapper = new Mapper\AstPsEndpoints();
        $endpoint = $endpointsMapper->findOneByField("sorcery_id", $endpointName);
        if (empty($endpoint)) {
            return null;
        }

        // User endpoint!
        $terminal = $endpoint->getTerminal();
        if ($terminal) {
            // Get caller user
            $user = $terminal->getUser();
            if (empty($user)) {
                $this->agi->error("Terminal %s has no user.", $terminal->getId());
                return null;
            }

            // Get caller extension
            $extension = $user->getExtension();
            if (empty($extension)) {
                $this->agi->error("User %s has no extension.", $user->getId());
                return null;
            }

            // Get Caller for this entity
            return $user;
        }

        // Friend Endpoint!
        $friend = $endpoint->getFriend();
        if ($friend) {
            return $friend;
        }

        // Retail Endpoint!
        $retail = $endpoint->getRetailAccount();
        if ($retail) {
            return $retail;
        }

        return null;
    }

}
