<?php

namespace Agi\Action;
use \IvozProvider\Model\Features;

/**
 * @class ExternalFriendCallAction
 *
 * @brief Manage outgoing external calls generated by a friendly trunk
 *
 */
class ExternalFriendCallAction extends ExternalCallAction
{
    protected $_number;

    protected $_checkACL;

    public function setDestination($number)
    {
        $this->_number = $number;
        return $this;
    }

    public function setCheckACL($acl)
    {
        $this->_checkACL = $acl;
        return $this;
    }

    public function process()
    {
        // Local variables
        $friend = $this->agi->getChannelCaller();
        $number = $this->_number;

        // Get company from the caller
        $company = $friend->getCompany();

        // Some feedback for asterisk cli
        $this->agi->notice("Processing External call from \e[0;36m%s [friend%d]\e[0;93m to %s",
            $friend->getName(), $friend->getId(), $number);

        // Check if dialed number has company's outbound prefix
        if (!$this->checkCompanyOutboundPrefix($number)) {
            $this->agi->error("Destination number %s without [company%d] outbound prefix",
                            $number, $company->getId());
            $this->agi->decline();
            return;
        }

        // Convert to E.164 format
        $e164number = $friend->preferredToE164($number);

        // Check if the diversion header contains a valid number
        $this->checkDiversionNumber($company, $e164number);

        // Check the user has this call allowed in its ACL
        if ($this->_checkACL && !$friend->isAllowedToCall($e164number)) {
            $this->agi->error("User is not allowed to call %s", $e164number);
            // Play error notification over progress
            if ($company->hasFeature(Features::PROGRESS)) {
                $this->agi->progress("ivozprovider/notAllowed");
            }
            $this->agi->decline();
            return;
        }

        // Check if outgoing call can be tarificated
        if (!$this->checkTarificable($e164number)) {
            $this->agi->error("Destination %s can not be billed.", $e164number);
            // Play error notification over progress
            if ($company->hasFeature(Features::PROGRESS)) {
                $this->agi->progress("ivozprovider/notBillable");
            }
            $this->agi->decline();
            return;
        }

        // Check if DDI has valid redirecting number
        if ($this->agi->getRedirecting('count')) {
            $diversionNum = $this->agi->getRedirecting('from-num');
            if(($diversionDDI = $company->getDDI($diversionNum))) {
                $ddi = $diversionDDI;
                // Diversion DDI is valid, convert Presented number format
                $callerIdNum = $friend->preferredToE164($this->agi->getCallerIdNum());
                $this->agi->setCallerIdNum($callerIdNum);
            }
        }

        if (!isset($ddi)) {
            // Allow identification from any company DDI
            $callerIdNum = $friend->preferredToE164($this->agi->getCallerIdNum());
            $companyDDIs = $friend->getCompany()->getDDIs();
            foreach ($companyDDIs as $companyDDI) {
                if ($callerIdNum === $companyDDI->getDDIE164()) {
                    $this->agi->notice("Friend \e[0;36m%s [friend%d]\e[0;93m presented origin matches company DDI %s [ddi%d].",
                             $friend->getName(), $friend->getId(), $companyDDI->getDDIE164(), $companyDDI->getId());
                    $ddi = $companyDDI;
                    $this->agi->setCallerIdNum($ddi->getDDIE164());
                    break;
                }
            }
        }

        // Use fallback outgoing DDI
        if (!isset($ddi) || !$ddi) {
            $ddi = $friend->getOutgoingDDI();
            if ($ddi) {
                $this->agi->notice("Using fallback DDI %d [ddi%s] for friend \e[0;36m%s [friend%d]\e[0;93m because %s does not match any DDI.",
                    $ddi->getDDIE164(), $ddi->getId(), $friend->getname(), $friend->getId(), $callerIdNum);
                $this->agi->setCallerIdNum($ddi->getDDIE164());
            }
        }

        // Update caller displayed number
        if (!$ddi) {
            $this->agi->error("Friend %s [friend%d] has not OutgoingDDI configured",  $friend->getName(), $friend->getId());
            $this->agi->decline();
            return;
        }

        // Check if DDI has recordings enabled
        $this->checkDDIRecording($ddi);
        // Check if DDI belong to platform
        $this->checkDDIBounced($e164number);


        // Call the PSJIP endpoint
        $this->agi->setVariable("DIAL_DST", "PJSIP/" . $e164number . '@proxytrunks');
        $this->agi->setVariable("DIAL_OPTS", "");
        $this->agi->setVariable("DIAL_TIMEOUT", "");
        $this->agi->redirect('call-world', $e164number);
    }
}
